/***************************************************************************
    messages.h

    This file is part of the CBE Dictionary package

    Copyright (c) 2001 Antoby - antoby@mail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    $Id: messages.h,v 1.3 2001/06/20 15:48:06 bbi Exp $

****************************************************************************/


#ifndef MESSAGES_H
#define MESSAGES_H

enum {
    MSG_FALSE=0,
    MSG_TRUE,
    MSG_SEPARATE_MEANINGS,
    MSG_LATIN_INPUT,
    MSG_LATIN_OUTPUT,
    MSG_HTML_OUTPUT,
    MSG_EXACT_MATCH,
    MSG_ENG_BUL_DAT,
    MSG_BUL_ENG_DAT,
    MSG_CHANGE_OPTION,
    MSG_SAVE_AND_QUIT,
    MSG_QUIT_WITHOUT_SAVE,
    MSG_INCORRECT_OPTION_INDEX,
    MSG_PROMPT_EN_BG_CHANGE,
    MSG_PROMPT_BG_EN_CHANGE,
    MSG_START_INTERACTIVE,
    MSG_CONFIG_SAVED,
    MSG_QUITTING,
};

char **select_language(void);

#endif
